/*
 * DynFunctionRegistry.C
 *
 *  Created on: Nov 29, 2010
 *      Author: Tobias Brosch
 */

#include <iostream>
#include <string>
#include <dlfcn.h>
#include "DynFunctionRegistry.h"


DynFunctionRegistry::DynFunctionRegistry() {

}

DynFunctionRegistry::DynFunctionRegistry(const std::string dirname) :
	dir(dirname) {

}

void DynFunctionRegistry::add(Function* f) {
	registry[f->get_name()] = f;
}

bool DynFunctionRegistry::is_known(const std::string fname) const {
	return registry.find(fname) != registry.end();
}

//Function* FunctionRegistry::get_function(std::string fname) {
//	return registry[fname];
//}

Function* DynFunctionRegistry::get_function(const std::string& fname) {
	std::map<std::string, Function*>::iterator it(registry.find(fname));
	Function *f = 0;
	if (it == registry.end()) {
		f = dynload(fname);
		if (f) {
			add(f);
			if (f->get_name() != fname)
				registry[fname] = f;
		}
	} else {
		f = it->second;
	}
	return f;
}




Function* DynFunctionRegistry::dynload(const std::string& name) {

	typedef Function* FunctionConstructor();
	std::string path(dir);
	if (path.size() > 0)
		path += "/";

	path += "lib";
	path += name;
	path += ".so";

	void* handle = dlopen(path.c_str(), RTLD_LAZY | RTLD_LOCAL);
	if (!handle) {
		return 0;
	}

	std::string constructFuncName("construct");
	constructFuncName += name;

	FunctionConstructor* constructor = (FunctionConstructor*) dlsym(handle,
			constructFuncName.c_str());
	if (!constructor) {
		dlclose(handle);
		return 0;
	}
	return constructor();
}
